#!/usr/bin/python
'''
@File: fusion_csvs.py

@Author: amp

@brief: Script para agrupar medidas de T y RH de varios .csv en uno solo

El script toma como argumentos de entrada una lista de los .csv que se quieren fusionar
y el nombre del archivo en el que se guardan los datos fusionados

Algoritmo: 
    1-Tomar la columna de tiempos de todos los .csv
    2-Agrupar las columnas de tiempo en un solo array
    3-Ordenar el array 
    4-Eliminar duplicados 
    5-Añadir datos de T y RH 

Restricciones:
    - El separador decimal debe ser '.'
    - El formato de timestamp debe ser igual en todos los .csv (%d/%m/%Y %H:%M')
'''
import pandas as pd
import sys
import argparse
from datetime import datetime
import csv

def fusion_csvs(argv):
    input_files = ''
    outputfile = 'csv_fusion.csv'
    date_time_format = '%d/%m/%Y %H:%M'
    #get arguments
    try:
        parser = argparse.ArgumentParser()
        parser.add_argument('-i', '--inputs', help='Input files', nargs='+', required=True)
        parser.add_argument('-o', '--output', help='Output name', nargs='+', required=False)
        args = parser.parse_args()
    except:
        print('test.py  -o <output files> -i <input files list>')
        sys.exit(2)


    input_files = args.inputs
    if( args.output != None):
        outputfile = args.output


    timestamp_array = []

    #Fusion, sort and remove duplicates of all timestamp columns from all files
    for csv_file in input_files:
        csv_content = pd.read_csv(csv_file, delimiter=';',usecols=["time","T","RH"])
        for timestamp in csv_content['time']:
            timestamp_array.append(datetime. strptime(timestamp, date_time_format))
    timestamp_array = sorted(timestamp_array)
    timestamp_array = list(dict.fromkeys(timestamp_array))
  
    #Open output file
    with open(outputfile, mode='w') as output:
        writer = csv.writer(output, delimiter = ';', lineterminator = '\n')
        
        csv_content = []      #All the contents of the input csv files is loaded in this matrix
        timestamp_matrix = [] #This matrix stores the timestamp column of each csv
        index = 0
        #Load csv information into 'csv_content' variable and timestamp column of each csv into 'timestamp_matrix'
        for i in range(len(input_files)):
            csv_content.append(pd.read_csv(input_files[i], delimiter=';',usecols=["time","T","RH"]))
            temp_timestamp_array = []
            for timestamp in csv_content[i]['time']:
                temp_timestamp_array.append(datetime. strptime(timestamp, date_time_format))
            timestamp_matrix.append(temp_timestamp_array)

        #Print csv header row
        row = ['timestamp']
        for filename in input_files:
            row.append(filename[:-4] + ' T')
            row.append(filename[:-4] + ' RH')
        writer.writerow(row)

        #Look for timestamp into timestamp_matrix. If timestamp_matrix[file] contains the timestamp, look for 
        # timestamp index, and lprint data to the output csv. If not, print 0s
        for t in timestamp_array:
            row = []
            time_str = t.strftime(date_time_format)
            row.append(time_str)         
            for i in range(len(input_files)):              
                if t in timestamp_matrix[i]:
                    index = timestamp_matrix[i].index(t)                                      
                    row.append(csv_content[i]['T'][index])
                    row.append(csv_content[i]['RH'][index])
                else:
                     row.append(0)
                     row.append(0)
        
            writer.writerow(row)

        output.flush()
        output.close()
    
             

if __name__ == "__main__":
    fusion_csvs(sys.argv[1:])    
