#!/usr/bin/python
'''
@File: group_by_date.py

@Author: pmg

@brief: Script para agrupar medidas de temperatura y humedad almacenadas en un .csv dentro de un mismo minuto.
              
Cuando se realizan log de medidas de temperatura y humedad se obtiene un .csv con dichas medidas en intervalos de 
tiempo inferiores al minuto. El presente script agrupa las medidas de cada minuto, calcula la media de las mimas
y genera un .csv con el resultado.

Restricciones:
    - El separador del .csv debe ser ';'
    - El punto decimal es el caracter '.'
    - El orden de las columnas es: Timestamp; Temperatura; RH
    - El formato del timestamp es HH:MM
'''

import csv
from statistics import mean
import sys
import getopt
from datetime import datetime

def main(argv):
    inputfile = ''
    outputfile = ''
    date_time_format = '%d/%m/%Y %H:%M'
    csv_header = ["time","T","RH"]

    #get arguments
    try:
        opts, args = getopt.getopt(argv, "hi:o:d:", ["ifile=", "ofile=","dateformat"])
    except getopt.GetoptError:
        print('test.py -i <inputfile> -o <outputfile> [-d <date time format>]')
        sys.exit(2)

    for opt, arg in opts:
        if opt == '-h':
            print('test.py -i <inputfile> -o <outputfile>')
            sys.exit()
        elif opt in ("-i", "--ifile"):
            inputfile = arg
        elif opt in ("-o", "--ofile"):
            outputfile = arg
        elif opt in ("-d", "--dateformat"):
            date_time_format = arg

    temperatures = {}
    humidities = {}
    with open(inputfile, mode='rU') as f:
        #Read csv
        reader = csv.reader(f, delimiter=';')  # dialect=csv.excel_tab?
        #Load temeperature and humidity data into arrays
        for n, row in enumerate(reader):
            if not n:
                # Skip header row (n = 0).
                continue
            t = datetime. strptime(row[0], date_time_format)
            temperature = row[1]
            humidity = row[2]
            if t not in temperatures:
                temperatures[t] = list()
                humidities[t] = list()
            temperatures[t].append(float(temperature))
            humidities[t].append(float(humidity))
    
    with open(outputfile, mode='w') as output:
        writer = csv.writer(output, delimiter = ';', lineterminator = '\n')
        writer.writerow(csv_header)
        for t in sorted(temperatures):
            writer.writerow([t.strftime(date_time_format),mean(temperatures[t]),mean(humidities[t])])
        output.flush()
        output.close()


if __name__ == "__main__":
    main(sys.argv[1:])